var ILOFileUtils = {
  getInstallDir: function() {
    return Components.classes["@mozilla.org/extensions/manager;1"]
                .getService(Components.interfaces.nsIExtensionManager)
                .getInstallLocation(EXT_ID)
                .getItemLocation(EXT_ID);
  },
  
  getExtensionFile: function(aPath) {
    var file = this.getInstallDir();
    aPath.split("/").forEach(function(path) {
      file.append(path);
    })
    
    return file;
  },
  
  readExtensionFile: function(aPath) {
    var file = this.getExtensionFile(aPath);
    return (file.exists() && file.isFile()) ? this.readFile(file) : null;
  },
  
  _unicodeConverter: null,
  
  get unicodeConverter() {
    if (!this._unicodeConverter) {
      var uc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
                         .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
      uc.charset = "UTF-8";
      
      this._unicodeConverter = uc;
    }
    
    return this._unicodeConverter;
  },
  
  safeUnicode: function(aString) {
    if (!/[^\r\n\x9\xA\xD\x20-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]/.test(aString))
      return aString;
    
    return aString.replace(/[^\r\n\x9\xA\xD\x20-\uD7FF\uE000-\uFFFD\u10000-\u10FFFF]/g, "");
  },
  
  writeFile: function(aFile, aData) {
    var chunk = this.unicodeConverter.ConvertFromUnicode(aData);
    
    var os = Components.classes["@mozilla.org/network/file-output-stream;1"]
                       .createInstance(Components.interfaces.nsIFileOutputStream);
    os.init(aFile, 0x02 | 0x08 | 0x20, 0755, 0);
    var result = os.write(chunk, chunk.length);
    os.close();
  },
  
  readFile: function(aFile) {
    var fileContents = "";
    
    var is = Components.classes["@mozilla.org/network/file-input-stream;1"]
                       .createInstance(Components.interfaces.nsIFileInputStream);
    is.init(aFile, 0x01, 0, is.CLOSE_ON_EOF);
    
    var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
                        .createInstance(Components.interfaces.nsIScriptableInputStream);
    sis.init(is);
    
    while(sis.available() > 0)
      fileContents += this.unicodeConverter.ConvertToUnicode(sis.read(sis.available()));
    
    is.close();
    sis.close();
    
    return this.safeUnicode(fileContents);
  }
}