// todo: Preferences module

/**
 * A wrapper around nsIPrefService
 * (built in exception handling; default value for getters)
 **/
function PrefsWrapper(aRoot) {
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  
  this.branch = prefService.getBranch(aRoot);
  this.branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  
  ["Int", "Char", "Bool"]
  .forEach(function(aType) {
    PrefsWrapper.prototype["get" + aType + "Pref"] = new Function(
      "aPrefName", "aDefValue",
        "try { return this.branch.get" + aType + "Pref(aPrefName);" + 
        "} catch(e) {} return (typeof aDefValue == 'undefined' ? null : aDefValue);");
    
    PrefsWrapper.prototype["set" + aType + "Pref"] = new Function(
      "aPrefName", "aValue",
        "try { return this.branch.set" + aType + "Pref(aPrefName, aValue);" + 
        "} catch(e) {}");
  })
}

/**
 *
 */
function PrefsContainer(aBranch, aPrefsCallbacks) {
  this._prefsWrapper = new PrefsWrapper(aBranch);
  this._prefsHash = {};
  this._prefsCallbacks = aPrefsCallbacks || {};
  
  this._init();
}

PrefsContainer.prototype = {
  _getCamelName: function(aName) {
    var parts = aName.split(".");
    var camelName = parts.shift();
    
    parts.forEach(function(aPart) {
      camelName += aPart.charAt(0).toUpperCase() + aPart.substring(1);
    })
    
    return camelName;
  },
  
  _init: function() {
    var branch = this._prefsWrapper.branch;
    var prefTypes = {};
    prefTypes[branch.PREF_INT] = "Int";
    prefTypes[branch.PREF_BOOL] = "Bool";
    
    branch.getChildList("", {})
    .forEach(function(aPrefName) {
      var prefType = branch.getPrefType(aPrefName);
      prefType = (prefType in prefTypes ? prefTypes[prefType] : "Char");
      
      var camelName = this._getCamelName(aPrefName);
      
      this._prefsHash[aPrefName] = {
        def: this._prefsWrapper["get" + prefType + "Pref"](aPrefName),
        type: prefType,
        camelName: camelName,
        func: (aPrefName in this._prefsCallbacks ?
                  new Function("aValue", "return " + this._prefsCallbacks[aPrefName] + "(aValue)") :
                  null)
      }
      
      this.__defineGetter__(
        camelName,
        function() {
          return this["_" + camelName];
        }
      );
      
      this.__defineSetter__(
        camelName,
        function(aValue) {
          return this._prefsWrapper["set" + prefType + "Pref"](aPrefName, aValue);
        }
      );
    }, this)
  },
  
  _setPrefValue: function(aPrefName) {
    var pref = this._prefsHash[aPrefName];
    if (pref) {
      this["_" + pref.camelName] = this._prefsWrapper["get" + pref.type + "Pref"](aPrefName, pref.def);
      
      if (pref.func)
        pref.func(this[pref.camelName]);
    }
  },
  
  init: function() {
    for (var prefName in this._prefsHash)
      this._setPrefValue(prefName);
    
    this._prefsWrapper.branch.addObserver("", this, false);
  },
  
  uninit: function() {
    try {
      this._prefsWrapper.branch.removeObserver("", this);
    } catch(e) {}
    
    for each (var pref in this._prefsHash)
      delete this["_" + pref.camelName];
  },
  
  observe: function(aSubject, aTopic, aData) {
    if (aTopic == "nsPref:changed") {
      this._setPrefValue(aData);
      //~ this.notifyListeners(...);
    }
  }
}